/*
 * Main.c
 *
 *  Created on: 15.01.2014
 *      Author: oe
 */

#include <DAVE3.h>			//Declarations from DAVE3 Code Generation (includes SFR declaration)

/* header of project specific types
---------------------------------------------------------------------------*/
#include <gen_define.h>

#include <co_canopen.h>
#include <co_401.h>
#include <usr_401.h>


/* list of local defined functions
---------------------------------------------------------------------------*/
static RET_T nmtInd(BOOL_T  execute, CO_NMT_STATE_T newState);
static void hbState(UNSIGNED8   nodeId, CO_ERRCTRL_T state,
        CO_NMT_STATE_T  nmtState);
static RET_T sdoServerReadInd(BOOL_T execute, UNSIGNED8 sdoNr, UNSIGNED16 index,
        UNSIGNED8   subIndex);
static RET_T sdoServerCheckWriteInd(BOOL_T execute, UNSIGNED8 node,
        UNSIGNED16 index, UNSIGNED8 subIndex, const UNSIGNED8 *pData);
static RET_T sdoServerWriteInd(BOOL_T execute, UNSIGNED8 sdoNr,
        UNSIGNED16 index, UNSIGNED8 subIndex);
static void pdoInd(UNSIGNED16);
static void pdoRecEvent(UNSIGNED16);
static void canInd(CO_CAN_STATE_T);
static void commInd(CO_COMM_STATE_EVENT_T);
static void ledGreenInd(BOOL_T);
static void ledRedInd(BOOL_T);

#include <dave3.h>

int main(void)
{

	DAVE_Init();

	/* additional hardware init not done by Dave Apps */
	p401HardwareInit();

	/* Flash the LED2.1 for a short moment
	 * Later it is used as CANopen 'Green' LED Indicator */
	IO004_ResetPin(IO004_Handle0);
	IO004_SetPin(IO004_Handle0);

	// Initialization of DAVE Apps
	if (codrvCanInit(250) != RET_OK)  {
		return(1);

	NVIC002_EnableIRQ(&NVIC002_Handle0);

	}
	if (coCanOpenStackInit(NULL) != RET_OK)  {
	    // printf("error init library\n");
	    return(1);
	}

	if (codrvTimerSetup(CO_TIMER_INTERVAL) != RET_OK)  {
	    return(2);
	}

    /* register event functions */
    if (coEventRegister_NMT(nmtInd) != RET_OK)  {
        return(3);
    }
    if (coEventRegister_ERRCTRL(hbState) != RET_OK)  {
        return(4);
    }
    if (coEventRegister_SDO_SERVER_READ(sdoServerReadInd) != RET_OK)  {
        return(5);
    }
    if (coEventRegister_SDO_SERVER_CHECK_WRITE(sdoServerCheckWriteInd) != RET_OK)  {
        return(6);
    }
    if (coEventRegister_SDO_SERVER_WRITE(sdoServerWriteInd) != RET_OK)  {
        return(7);
    }
    if (coEventRegister_PDO(pdoInd) != RET_OK)  {
        return(8);
    }
    if (coEventRegister_PDO_REC_EVENT(pdoRecEvent) != RET_OK)  {
        return(9);
    }
    if (coEventRegister_LED_GREEN(ledGreenInd) != RET_OK)  {
        return(10);
    }
    if (coEventRegister_LED_RED(ledRedInd) != RET_OK)  {
        return(11);
    }
    if (coEventRegister_CAN_STATE(canInd) != RET_OK)  {
        return(12);
    }
    if (coEventRegister_COMM_EVENT(commInd) != RET_OK)  {
        return(13);
    }

    /* --- 401 specific ------------------------------------------------*/
    /* profile 401 HW acess registration */
    if (co401Init() != RET_OK)  {
        // printf("error init 401 profile handler\n");
        return(14);
    }

    if (coEventRegister_401(
                byte_in_piInd,      /* digital in  HW API function   */
                byte_out_piInd,     /* digital out HW API function   */
                analog_in_piInd,    /* the analog in HW API function */
                analog_out_piInd    /* analog out HW API function    */
                ) != RET_OK)  {
       // printf("registration of 401 HW acess functions failed\n");
        return(14);
    }
    /* -----------------------------------------------------------------*/
    codrvCanEnable();
	while (1)  {
		coCommTask();
		co401Task();
	}
}


/*********************************************************************
The following are definitions of communication indication functions.
The application can use these functions to process communication events
application specific. This example, implementing the CiA 401 profile,
is only using the indications to display it using simple printf()
messages.
**********************************************************************/
/*********************************************************************/
RET_T nmtInd(
    BOOL_T  execute,
    CO_NMT_STATE_T  newState
)
{
	// printf("USR: nmtInd: New Nmt state %d - execute %d\n", newState, execute);

	return(RET_OK);
}


/*********************************************************************/
void pdoInd(
    UNSIGNED16  pdoNr
)
{
	// printf("USR: pdoInd: pdo %d received\n", pdoNr);
}


/*********************************************************************/
void pdoRecEvent(
    UNSIGNED16  pdoNr
)
{
	// printf("USR: pdoRecEvent: pdo %d time out\n", pdoNr);
}


/*********************************************************************/
void hbState(
    UNSIGNED8   nodeId,
    CO_ERRCTRL_T state,
    CO_NMT_STATE_T  nmtState
)
{
	// printf("USR: hbInd: HB Event %d node %d nmtState: %d\n", state, nodeId, nmtState);

	return;
}



/*********************************************************************/
static RET_T sdoServerReadInd(
    BOOL_T      execute,
    UNSIGNED8   sdoNr,
    UNSIGNED16  index,
    UNSIGNED8   subIndex
)
{
	// printf("USR: sdo server read ind: exec: %d, sdoNr %d, index %x:%d\n",
	//  execute, sdoNr, index, subIndex);

	return(RET_OK);
}


/*********************************************************************/
static RET_T sdoServerCheckWriteInd(
    BOOL_T      execute,
    UNSIGNED8   sdoNr,
    UNSIGNED16  index,
    UNSIGNED8   subIndex,
    const UNSIGNED8 *pData
)
{
	// printf("USR: sdo server check write ind: exec: %d, sdoNr %d, index %x:%d\n",
	//  execute, sdoNr, index, subIndex);

	return(RET_OK);
}


/*********************************************************************/
static RET_T sdoServerWriteInd(
    BOOL_T      execute,
    UNSIGNED8   sdoNr,
    UNSIGNED16  index,
    UNSIGNED8   subIndex
)
{
	// printf("USR: sdo server write ind: exec: %d, sdoNr %d, index %x:%d\n",
	//  execute, sdoNr, index, subIndex);

	return(RET_OK);
}

UNSIGNED8 ledval;
/* helper function */
void setLED_x(UNSIGNED8 val, int low)
{
	if(!low) {
		val &= 0x0f;
		val |= (ledval & 0xf0);
	}
	else {
		val &= 0xf0;
		val |= (ledval & 0x0f);
	}
	ledval = val;
	//	setLED(val);
}

/*********************************************************************/
static void canInd(
CO_CAN_STATE_T  canState
)
{
	switch (canState)  {
		case CO_CAN_STATE_BUS_OFF:
			// printf("USR: CAN: Bus Off\n");
			setLED_x((UNSIGNED8)(3), 0);
			break;
		case CO_CAN_STATE_BUS_ON:
			// printf("USR: CAN: Bus On\n");
			setLED_x((UNSIGNED8)(0), 0);
			break;
		case CO_CAN_STATE_PASSIVE:
			// printf("USR: CAN: Passiv\n");
			setLED_x((UNSIGNED8)(1), 0);
			break;
		case CO_CAN_STATE_UNCHANGED:
			// printf("USR: CAN: unchanged\n");
			// setLED_x((UNSIGNED8)(), 0);
			break;
		default:
			break;
	}
}


/*********************************************************************/
static void commInd(
    CO_COMM_STATE_EVENT_T   commEvent
)
{

switch (commEvent)  {
    case CO_COMM_STATE_EVENT_BUS_OFF:
        // printf("COMM-Event Bus Off\n");
		setLED_x((UNSIGNED8)(0xc0), 1);
        break;
    case CO_COMM_STATE_EVENT_BUS_OFF_RECOVERY:
        // printf("COMM-Event Bus Off\n");
        break;
    case CO_COMM_STATE_EVENT_BUS_ON:
        // printf("COMM-Event Bus On\n");
		setLED_x((UNSIGNED8)(0x00), 1);
        break;
    case CO_COMM_STATE_EVENT_PASSIVE:
        // printf("COMM-Event Bus Passive\n");
		setLED_x((UNSIGNED8)(0x80), 1);
        break;
    case CO_COMM_STATE_EVENT_ACTIVE:
        // printf("COMM-Event Bus Active\n");
        break;
    case CO_COMM_STATE_EVENT_CAN_OVERRUN:
        // printf("COMM-Event CAN Overrun\n");
        break;
    case CO_COMM_STATE_EVENT_REC_QUEUE_FULL:
        // printf("COMM-Event Rec Queue Full\n");
        break;
    case CO_COMM_STATE_EVENT_REC_QUEUE_OVERFLOW:
        // printf("COMM-Event Rec Queue Overflow\n");
        break;
    case CO_COMM_STATE_EVENT_TR_QUEUE_FULL:
        // printf("COMM-Event Tr Queue Full\n");
        break;
    case CO_COMM_STATE_EVENT_TR_QUEUE_OVERFLOW:
        // printf("COMM-Event Tr Queue Empty\n");
        break;
    case CO_COMM_STATE_EVENT_TR_QUEUE_EMPTY:
        // printf("COMM-Event Tr Queue Empty\n");
        break;
    default:
        break;
}

}


/* define colors */
#define RED     ATTBOLD FGRED
#define GREEN   ATTBOLD FGGREEN

/* define LED index */
#define LGREEN  0
#define LRED    1

static int leds[2] = { 0 , 0 };

/*********************************************************************/
static void showLEDs(void)
{
#if 0
printf("> " GREEN "%c  " RED "%c"  ATTRESET "\r",
    leds[LGREEN] ? 'O' : ' ',
    leds[LRED]   ? 'O' : ' ');
//  fflush(stdout);
#endif
}

/*********************************************************************/
static void ledGreenInd(
    BOOL_T  on
)
{
	// printf("GREEN: %d\n", on);
	leds[LGREEN] = on;
	showLEDs();

	/* hardware control on the XMC4200 Board.
	 * P2.1 is used to control the LED.
	 * Low level on Pn 2.1 switches LED on.
	 */
	if (on == CO_TRUE) {
		IO004_ResetPin(IO004_Handle0);
	} else {
		IO004_SetPin(IO004_Handle0);
	}
}


/*********************************************************************/
static void ledRedInd(
    BOOL_T  on
)
{
	// printf("RED: %d\n", on);
	leds[LRED] = on;
	showLEDs();
}

/* vim: set ts=4 sw=4 spelllang=en : */
