/*
* co_pdo.h - contains defines for pdo services
*
* Copyright (c) 2012 emtas GmbH
*
*-------------------------------------------------------------------
* $Id: co_pdo.h 4300 2013-08-12 13:56:42Z  $

*-------------------------------------------------------------------
*
* Changelog:
*
*
*/

/**
* \brief defines for pdo service
*
* \file co_pdo.h - contains defines for pdo service
*/

#ifndef CO_PDO_H
#define CO_PDO_H 1

# if defined(CO_PDO_TRANSMIT_CNT) || defined(CO_PDO_RECEIVE_CNT)


#include <co_datatype.h>

/**
* PDO mapping entry (one mapping entry)
*/
typedef struct {
	void		*pVar;		/**< pointer to variable */
	UNSIGNED8	len;		/**< number of bytes for variable */
	BOOL_T		numeric;	/**< numeric flag for byte swapping */
	UNSIGNED32	val;		/**< OD value */
} PDO_MAP_ENTRY_T;

/**
* PDO mapping table (mapping entries for one PDO) */
typedef struct {
	UNSIGNED8		mapCnt;		/**< number of mapping entries */
	PDO_MAP_ENTRY_T	mapEntry[CO_MAX_MAP_ENTRIES]; /**< Mapping entries */
} PDO_MAP_TABLE_T;


/** \brief function pointer to PDO indication
 * \param pdoNr - PDO number
 * 
 * \return void
 */
typedef void (* CO_EVENT_PDO_T)(UNSIGNED16); /*lint !e960 customer specific parameter names */


/* function prototypes */
RET_T coPdoTransmitInit(UNSIGNED16 pdoNr, UNSIGNED8	transType,
	UNSIGNED16 inhibit, UNSIGNED16	eventTime, UNSIGNED8 syncStartVal,
	const PDO_MAP_TABLE_T *mapTable);
RET_T coPdoReceiveInit( UNSIGNED16 pdoNr, UNSIGNED8 transType,
	UNSIGNED16 inhibit, UNSIGNED16 eventTime, const PDO_MAP_TABLE_T *mapTable);

RET_T coPdoReqNr(UNSIGNED16 pdoNr, UNSIGNED8 flags);
RET_T coPdoReqObj(UNSIGNED16 index, UNSIGNED8 subIndex, UNSIGNED8 flags);

BOOL_T coPdoObjIsMapped(UNSIGNED16 pdoNr, UNSIGNED16 index, UNSIGNED8 subIndex);

RET_T coEventRegister_PDO(CO_EVENT_PDO_T pFunction);
RET_T coEventRegister_PDO_SYNC(CO_EVENT_PDO_T pFunction);
RET_T coEventRegister_PDO_REC_EVENT(CO_EVENT_PDO_T pFunction);

# endif /* defined(CO_PDO_TRANSMIT_CNT) || defined(CO_PDO_RECEIVE_CNT) */
#endif /* CO_PDO_H */

