/*
* co_timer.h - contains defines for timer
*
* Copyright (c) 2012 emtas GmbH
*
*-------------------------------------------------------------------
* $Id: co_timer.h 4579 2013-09-25 11:58:40Z  $

*-------------------------------------------------------------------
*
* Changelog:
*
*
*/

/**
* \brief defines for timer
*
* \file co_timer.h - contains defines for timer
*/

#ifndef CO_TIMER_H
#define CO_TIMER_H 1


/* datatypes */

/**
* timer attributes
*/
typedef enum {
	CO_TIMER_ATTR_ROUNDUP,			/**< round up given timer value */
	CO_TIMER_ATTR_ROUNDUP_CYCLIC,	/**< round up and start timer again */
	CO_TIMER_ATTR_ROUNDDOWN,		/**< round down given timer value */
	CO_TIMER_ATTR_ROUNDDOWN_CYCLIC	/**< round down and start timer again */
} CO_TIMER_ATTR_T;


/**
* timer structure 
*/
typedef struct co_timer {
	struct co_timer	*pNext;			/**< pointer to next timer */
	UNSIGNED32		actTicks;		/**< actual timer ticks */
	UNSIGNED32		ticks;			/**< calculated timer ticks */
	void		(*pFct)(void *);	/**< pointer to own function */
	void			*pData;			/**< pointer for own data */
	CO_TIMER_ATTR_T	attr;			/**< timer attributes */
} xTimer;
typedef struct co_timer	CO_TIMER_T;


/* function prototypes */

void	coTimerInit(UNSIGNED32	timerVal);
RET_T	coTimerStart(CO_TIMER_T *pTimer, UNSIGNED32 timerTime,
		void ptrToFct(void *p), void *pData, CO_TIMER_ATTR_T timerAttributes);
RET_T	coTimerStop(CO_CONST CO_TIMER_T *pTimer);
BOOL_T	coTimerIsActive(CO_CONST CO_TIMER_T *pTimer);
void	coTimerTick(void);
void	coTimerAttrChange(CO_TIMER_T *pTimer, CO_TIMER_ATTR_T timerAttributes);

#endif /* CO_TIMER_H */
